"""Flask middleware definitions. This is also where template filters are defined.

To be imported by the application.current_app() factory.
"""
from typing import Union, Type, Tuple, Optional
from logging import getLogger
from markupsafe import Markup
from werkzeug.security import check_password_hash
from flask import current_app, jsonify, Response
from gammu_smsd_api.extensions import http_basic_auth


LOG = getLogger(__name__)


@http_basic_auth.verify_password  # type: ignore [misc]
def verify_password(username: str, password: str) -> Optional[str]:
    users = current_app.config.get('USERS', {})

    found_password = users.get(username)

    if found_password and check_password_hash(found_password, password):
        return username

    return None


# Setup default error templates.
@current_app.errorhandler(400)
@current_app.errorhandler(403)
@current_app.errorhandler(404)
@current_app.errorhandler(500)
def error_handler(e: Union[Type[Exception], int]) -> Tuple[Response, int]:
    code = getattr(e, 'code', 500)  # If 500, e == the exception.
    return jsonify({'message': 'Error {}'.format(code)}), code


# Template filters.
@current_app.template_filter()
def whitelist(value: str) -> Markup:
    """Whitelist specific HTML tags and strings.
    Positional arguments:
    value -- the string to perform the operation on.
    Returns:
    Markup() instance, indicating the string is safe.
    """
    translations = {
        '&amp;quot;': '&quot;',
        '&amp;#39;': '&#39;',
        '&amp;lsquo;': '&lsquo;',
        '&amp;nbsp;': '&nbsp;',
        '&lt;br&gt;': '<br>',
    }
    escaped = str(Markup.escape(value))  # Escapes everything.
    for k, v in translations.items():
        escaped = escaped.replace(k, v)  # Un-escape specific elements using str.replace.
    return Markup(escaped)  # Return as 'safe'.
